//Funzione con gli algoritmi di riduzione
function [Dr] = modred(DD,elim,method)

// Ouput variables initialisation (not found in input variables)
Dr=[];

// Display mode
mode(0);

// Display warning for floating point exception
ieee(1);

Dr = DD;
ns = size(DD.A,1);
if ns==0 then
  return;
end;
keep = 1:ns;

//Seleziono gli indici da conservare
L = [];
j=0;
for i=1:ns,
  if (elim(i) == %F)
    j=j+1;
    L(j)=i;
  end 
end

keep=L;

//Seleziono gli indici da eliminare
L = [];
j=0;
for i=1:ns,
  if (elim(i) == %T)
    j=j+1;
    L(j)=i;
  end 
end

nokeep=L;

a=DD.A;
b=DD.B;
c=DD.C;
d=DD.D;

select method
  case "MatchGain" then
    // Match gain: divido in x1, da conservare, e x2, da eliminare
    a11 = a(keep,keep);
    a12 = a(keep,nokeep);
    a21 = a(nokeep,keep);
    a22 = a(nokeep,nokeep);
    b1 = b(keep,:);
    b2 = b(nokeep,:);
    c1 = c(:,keep);
    c2 = c(:,nokeep);
    n2 = size(nokeep,1);

    // Creo le matrici ridotte
    tolsing = %eps^0.75;
    if (DD(7) == 'd')
      // Sistema tempo-discreto
      a22 = a22 - eye(n2,n2);
    end;
    [l,u,p] = lu(a22);
    norm11 = norm(a11,1);
    %v03 = bool2s(%f);
    if bool2s(norm(a22,1)>(tolsing*norm11)) then %v03 = bool2s(mtlb_rcond(u)>tolsing);end;
    if %v03 then
      
      A21 = u\(l\(p*a21));
      B2 = u\(l\(p*b2));
      ar = a11 - a12 * A21;
      br = b1 - a12 * B2;
      cr = c1 - c2 * A21;
      dr = d - c2 * B2;
    else
      
      [u,s,v] = svd(a22);
      s = diag(s(1:n2,1:n2));
      nnz = sum(bool2s(mtlb_logic(s,">",tolsing*mtlb_max([s;norm11],"m"))),"m");
      u = u(:,mtlb_imp(1,nnz));
      v = v(:,mtlb_imp(1,nnz));
      s = s(mtlb_imp(1,nnz));
      A21 = (diag(1 ./s)*u')*a21;
      B2 = (diag(1 ./s)*u')*b2;
      A12 = a12 * v;
      C2 = c2 * v;
      ar = a11 - A12 * A21;
      br = b1 - A12 * B2;
      cr = c1 - C2 * A21;
      dr = d - C2 * B2;
    end;

  case "Truncate" then
    // Elimino gli stati selezionati
    ar = a(keep,keep);
    br = b(keep,:);
    cr = c(:,keep);
    dr = d;
end;

// Preparo le matrici da mandare in uscita
Dr.A = ar;
Dr.B = br;
Dr.C = cr;
Dr.D = dr;
Dr.X0 = zeros(size(keep,1),1);
endfunction
