//Funzione principale del toolbox
function [slbr,slb] = mor();

// Display mode
mode(0);

// Display warning for floating point exception
ieee(1);

 //Faccio scegliere all'utente se vuole lavorare in tempo-continuo o in tempo-discreto
 tipo=buttondialog("Specifica il tipo del sistema?","Continuo|Discreto");

//Controllo se l'utente ha scelto il tempo continuo o il tempo discreto;
if (tipo == '1')
 tipo = 'c';
 nome = 'continuo';
else
 tipo = 'd';
 nome = 'discreto';
end;


//Leggo i parametri del sistema continuo
if (tipo == 'c')
 txt=['A';'B';'C';'D'];
 sig=x_mdialog('Inserisci i parametri del sistema tempo-'+nome,txt,['A';'B';'C';'D']);
 A=evstr(sig(1));
 B=evstr(sig(2));
 C=evstr(sig(3));
 D=evstr(sig(4));
 
//Leggo i parametri del sistema discreto
else
 txt=['A';'B';'C';'D';'Tempo di campionamento'];
 sig=x_mdialog('Inserisci i parametri del sistema tempo-'+nome,txt,['A';'B';'C';'D';'.1']);
 A=evstr(sig(1));
 B=evstr(sig(2));
 C=evstr(sig(3));
 D=evstr(sig(4));
 tc=evstr(sig(5));   
end;

if (rank(cont_mat(A,B)) < size(A,1))
  buttondialog("Attenzione: il sistema non  completamente controllabile.","OK","error");
  return;
end

if (rank(obsv_mat(A,C)) < size(A,1))
  buttondialog("Attenzione: il sistema non  completamente osservabile.","OK","error");
  return;
end

//Creo il sistema
sl=syslin(tipo,A,B,C,D);

//Estraggo il sottosistema stabile
[sl_unstable,sl_stable] = dtsi_mod(sl);

//Calcolo del dimensioni del sistema completo (parte stabile + parte instabile)
TotalModes = size(sl.a,1);
if (sl_stable == 0)
  StableModes = 0;
else
  StableModes = size(sl_stable.a,1);
end

UnstableModes = TotalModes - StableModes;

if (StableModes == 0)
  buttondialog("Il sistema contiene solo modi instabili. Impossibile bilanciare un sistema completamente instabile!","OK","error");
  return;
end

if (UnstableModes > 0)
  if (UnstableModes == 1)
    msg = 'Attenzione: il sistema contiene 1 modo instabile.';
  else
    msg = sprintf('Attenzione: il sistema contiene %i modi instabili.',UnstableModes);
  end
  buttondialog(msg,"OK","warning");
end

//Bilancio il sottosistema stabile
slbS=balreal(sl_stable);


FirstStableMode = UnstableModes+1;

//Inizializzo a 0 il sistema bilanciato finale
slb = sl;
slb.a = zeros(TotalModes,TotalModes);
slb.b = zeros(TotalModes,1);
slb.c = zeros(1,TotalModes);

//Aggiungo i modi stabili al sistema bilanciato
if(StableModes > 0)
  tmp = slb.a;
  tmp(FirstStableMode:TotalModes,FirstStableMode:TotalModes)=slbS.a;
  slb.a = tmp;

  tmp = slb.b;
  tmp(FirstStableMode:TotalModes)=slbS.b;
  slb.b = tmp;

  tmp = slb.c;
  tmp(FirstStableMode:TotalModes)=slbS.c;
  slb.c = tmp;
end

//Aggiungo i modi instabili al sistema bilanciato
if(UnstableModes > 0)
  tmp = slb.a;
  tmp(1:UnstableModes,1:UnstableModes)=sl_unstable.a;
  slb.a = tmp;

  tmp = slb.b;
  tmp(1:UnstableModes)=sl_unstable.b;
  slb.b = tmp;

  tmp = slb.c;
  tmp(1:UnstableModes)=sl_unstable.c;
  slb.c = tmp;
end

//Calcolo il gramiano di controllabilit
//E' superfluo il calcolo del gramiano di osservabilit, poich
//dopo il bilanciamento il gramiano di osservabilit  coincidente con il gramiano di controllabilit
Wc=clean(ctr_gram(slbS));
//Wo=clean(obs_gram(slbS));


//Estraggo gli autovalori dal gramiano ed inserisco Inf per indicare i modi instabili
Gc(1:TotalModes) = %inf;
Gc(UnstableModes+1:TotalModes) = diag(Wc);

//Converto il vettore Gramiano in vettore di stringhe
L = [];
for i=1:size(Gc,1),
  L(i)=string(Gc(i));
end

//Avvio un ciclo infinito di riduzione per permettere all'utente di eseguire 
//tutte le riduzioni che vuole sul modello creato
counter = 0; //Porto il conto di quante riduzioni si effettuano sullo stesso sistema
while 1
  counter = counter + 1;
  index=x_choose(L,['Seleziona il valore sotto il quale effettuare il taglio (i valori ad Inf corrispondono a modi instabili).']);
  
  //Controllo se l'utente ha selezionato il primo autovalore, ovvero vuole ridurre ad ordine 0 il sistema
  if (index == 1)
    buttondialog("Impossibile ridurre il sistema ad ordine 0!","OK","error");
    counter = counter - 1;
    continue;
  end;
  
  //Controllo se l'utente ha selezionato un modo instabile
  if (index > 0 & L(index) == 'Inf')
    buttondialog("Impossibile selezionare un modo instabile come valore di taglio!","OK","error");
    counter = counter - 1;
    continue;
  end;  

  //Controllo se l'utente non ha selezionato alcun valore di taglio
  if (index == 0)
    choise = buttondialog("Non  stato selezionato il valore di taglio. Se si sceglie di continuare il modello non verr ridotto!","Continua|Indietro","warning");
    if (choise == '1')
      soglia = -1;
    else
      counter = counter - 1;
      continue;
    end
  end

  //Faccio scegliere all'utente il metodo di riduzione
  method=buttondialog("Specifica il metodo di riduzione?","Truncate|Match Gain");
  if (method == '1')
    method = 'Truncate';
  else
    method = 'MatchGain';
  end;
  

  if (index > 0)
    //Identifico il valore di soglia.
    soglia = Gc(index);
  end;

  elim = (Gc <= soglia);

  //Effettuo la riduzione
  slbr = modred(slb,elim,method);

  //Stampo il grafico della risposta al gradino
  if (tipo == 'c')    //Tempo continuo
    t=linspace(0,10,500);
    stepresp=csim('step',t,slb);
    stepresp_rid=csim('step',t,slbr);
    xset('window',counter); xbasc(); // create and clear window for subplots
    subplot(2,1,1);
    plot2d(t,stepresp, 1); xgrid();
    plot2d(t,stepresp_rid, 2);
    xtitle('Step response','','Amplitude (m)');

  else                //Tempo discreto
    u = ones(1:tc:10);
    stepresp=flts(u,slb);
    stepresp_rid=flts(u,slbr);
    xset('window',counter); xbasc(); // create and clear window for subplots
    subplot(2,1,1);
    plot2d2("onn",(1:size(u,'c'))',stepresp',1); xgrid();
    plot2d2("onn",(1:size(u,'c'))',stepresp_rid',2);
    xtitle('Step response','','Amplitude (m)');
  end;
  
  //Calcolo l'indice di qualit
  J=0;
  tmp=0;
  for i=1:size(stepresp,2),
    tmp = stepresp_rid(i) - stepresp(i);
    J = J + tmp^2;
  end

  J = iconvert(J,14);   //Converto in intero il fattore di qualit

  //Stampo la legenda
  FullOrder = sprintf('Full (dimension %i)',size(Gc));
  if (index == 0) //Controllo se il sistema non  stato ridotto
    index = size(Gc) + 1;
  end
  ReducedOrder = sprintf('%s-Reduced (dimension %i)',method,index - 1);
  Js = string(J);
  Quality = sprintf('Indice di Qualit: J=%s',Js);
  legends([FullOrder;ReducedOrder;Quality],[[1;1],[2;1],[0;0]], opt="lr");

  //Stampo la risposta in frequenza
  subplot(2,1,2);  
  bode([slb;slbr],0.01,10);
  
  //Chiedo se si vuole effettuare una nuova riduzione sul sistema
  choise=buttondialog("Vuoi effettuare una nuova riduzione su questo sistema?","Si|No");
  if (choise == '2')
    break;
  end
  
end  

endfunction
